	PROGRAM TRANSCK
C       TRANSMISSIVITY OF A HOMOGENEOUS AND ISOTHERMAL COLUMN
C       OF CO2 USING THE CORRELATED-K DISTRIBUTION METHOD
C       AND VARIABLE SPECTRAL RESOLUTION
C 
C       THE PRESSURE OF THE COLUMN IS 1 ATMOSPHERE
C
	IMPLICIT NONE
C
	INTEGER NGAUSS, NNU, NTEMP
      	PARAMETER(NGAUSS=7,NTEMP=16,NNU=17)
C DECLARATION OF THE VARIABLES
	INTEGER IGAUSS, INU, ITEMP, I1
      	REAL*8 XCO2, L1, T, QC
	REAL*8 CT1, CT2, KSTAR
	REAL*8 K1(1:NTEMP,1:NNU,1:NGAUSS)
	REAL*8 DNU1(1:NNU), SIG1(1:NNU), STAU(1:NNU)
	REAL*8 TEMP(1:NTEMP)
      	REAL*8 AMEGA(1:NGAUSS)
      	CHARACTER*72 NAME1, NAME2
	EXTERNAL QC
C GAUSS 7-PTS QUADRATURE COEFFICIENTS
      	DATA AMEGA/
     &	0.45000000E-01,
     &	0.24500000E+00,
     &	0.32000000E+00,
     &	0.24500000E+00,
     &	0.56111111E-01,
     &	0.51248583E-01,
     &	0.37640306E-01/
C TEMPERATURE AND MOLAR FRACTION OF CALCULATION
	DATA TEMP / 300., 375., 425., 500., 600., 700., 800., 900.,
     &	            1100., 1300., 1500., 1700., 1900.,
     &	            2100., 2300., 2500. /
C ENTERING THE DATA
	WRITE (6,*) 'ENTER XCO2, T (K), L (CM) :'
	READ (5,*) XCO2, T, L1
	NAME1 = 'CKCO2'
	WRITE (6,*) 'ENTER THE NAME OF THE RESULT FILE :'
	READ (5,'(A72)') NAME2
C READING THE CK DATA FILE
      	OPEN(UNIT=1,FILE=NAME1,STATUS='OLD')
	DO INU=1, NNU
	READ (1,*) SIG1(INU), DNU1(INU)
	  DO IGAUSS=1, NGAUSS
	    READ (1,*)
	    READ (1,2000) (K1(ITEMP,INU,IGAUSS),ITEMP=1,NTEMP)
	  ENDDO
	ENDDO
2000	FORMAT (3(5(1PE12.6,1X),/),1PE12.6)
	CLOSE (1)
C INTERPOLATION IN TEMPERATURE
	IF ((T.LT.TEMP(1)).OR.(T.GT.TEMP(NTEMP))) THEN
	  WRITE (6,*) 'ERROR : THE TEMPERATURE IS EITHER TOO LOW'
	  WRITE (6,*) 'OR TOO HIGH'
	  STOP
	ENDIF
	I1 = 1
100	CONTINUE
	IF (T.GT.TEMP(I1+1)) THEN 
	  I1 = I1 + 1
	  GOTO 100
	ENDIF
	CT1 = (TEMP(I1+1)-T)/(TEMP(I1+1)-TEMP(I1))
	CT2 = (T-TEMP(I1))/(TEMP(I1+1)-TEMP(I1))
C CALCULATING THE TRANSMISSIVITY
     	OPEN(UNIT=7,FILE=NAME2)
	DO INU=1, NNU
	  STAU(INU) = 0.D0
	  DO IGAUSS=1, NGAUSS
	    KSTAR = CT1*K1(I1,INU,IGAUSS) +
     &	            CT2*K1(I1+1,INU,IGAUSS)
	    KSTAR = KSTAR*XCO2/(T*QC(T))
	    STAU(INU) = STAU(INU)+ AMEGA(IGAUSS)*DEXP(-KSTAR*L1)
	  ENDDO
	  WRITE (7,1000) DNU1(INU), SIG1(INU), STAU(INU)
	ENDDO
	CLOSE (7)
1000	FORMAT(0PF5.1,1X,0PF6.1,1X,1PE10.4)     
	END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C PARTITION FUNCTION OF CO2 C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      	REAL*8 FUNCTION QC(T)
C
	IMPLICIT NONE
C DECLARATION OF THE VARIABLES
	INTEGER K, IK
	REAL*8 CC, T, RES
      	REAL*8 CCO2A1(4),CCO2B1(4),CO2A(4),CO2B(4)
C DATAS
      	DATA CCO2A1/-2.154173478E0,0.9670123486E0,-0.8082674825E-03,
     &	0.2806562032E-05/
      	DATA CCO2B1/-351.7866975E0,2.779333185E0,-0.3673680835E-02,
     &	0.4090051841E-05/
C CASE OF CO2 (4TH DEGREE POLYNOM FITTED ON EXACT CALC FROM ROTHMAN'S
C LEVELS. 1% ACCURACY IN THE 150-3000 K RANGE)
      	DO IK=1,4
      	  CO2A(IK)=CCO2A1(IK)
    	  CO2B(IK)=CCO2B1(IK)
	ENDDO
      	CC=1./T
      	RES=0.
      	DO K=1,4
      	  CC=CC*T
      	  IF(T.LT.405.) THEN
            RES=RES+CO2A(K)*CC
      	  ELSE
      	    RES=RES+CO2B(K)*CC
      	  ENDIF
	ENDDO
      	QC=RES
      	RETURN
      	END
