C**********************************************************************
C
C	CALCULATION OF RADIATIVE INTENSITY AND TRANSMISSIVITY OF
C       A NONISOTHERMAL AND NONHOMOGENEOUS MIXTURE COLUMN OF H2O-CO2-CO-
C	TRANSPARENT GAS-PARTICLES DISCRETIZED INTO N HOMOGENEOUS AND 
C       ISOTHERMAL ELEMENTS
C
C	WITH THE MALKMUS STATISTICAL NARROW-BAND MODEL AND 
C	THE CURTIS-GODSON APPROXIMATION AND A 25 cm-1 SPECTRAL RESOLUTION
C
C	THE INPUTS (FILE 'SNBINPUT') :
C	*****************************
C
C	- N : TOTAL NUMBER OF ELEMENTS
C	- FOR I VARYING FROM 1 TO N :
C	 LENGTH(CM),TEMPERATURE(K),PRESSURE(ATM),XH2O,XCO2,XCO,XSOOT
C	 (XSPEC = MOLAR FRACTION OF SPECIES SPEC=H2O,CO2 or CO)
C	 (XSOOT = VOLUME FRACTION OF PARTICLES IN THE MIXTURE)
C        (EACH LINE CORRESPONDS TO ONE ELEMENT I)
C	 (THE DIRECTION OF PROPAGATION IS FROM N TO 1)
C
C	THE OUTPUTS ARE :
C	*****************
C
C	FILE 'SNBINTENS' :
C	------------
C
C	- WAVE NUMBER (CM-1), SPECTRAL RADIATIVE INTENSITY AT 
C	  THE EXIT OF THE COLUMN  (IN W.M-2.STR-1.(CM-1)-1)
C	- TOTAL RADIATIVE INTENSITY (IN W.M-2.STR-1)
C
C	FILE 'SNBTRANS' :
C	-------------
C
C	- WAVE NUMBER (CM-1), TRANSMISSIVITY
C
C	THE OTHER FILES REQUIRED FOR THIS PROGRAM ARE :
C	***********************************************
C
C	- SNBWN : CONTAINS THE WAVE NUMBERS FOR WHICH CALCULATIONS ARE
C	  CARRIED OUT AND SPECTRAL WIDTHS BOTH IN cm-1
C         THE PROGRAM IS STOPPED WHEN A NEGATIVE WAVE NUMBER IS 
C         ENCOUNTERED
C         THE WAVE NUMBERS MUST BE WITHIN THE RANGE 150-9300 cm-1
C
C	- SNBH2O : H2O SNB PARAMETERS
C	- SNBCO2 : CO2 SNB PARAMETERS
C	- SNBCO  : CO  SNB PARAMETERS
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-C,E-H,O-Z)
      LOGICAL LICO,LICO2,LIH2O
      REAL KCO,KC,KH
      REAL *8 DWVNB
      PARAMETER (NMAX=100,NWVMAX=450)
      COMMON/PHYS/XD(NMAX),T(NMAX),P(NMAX),XH2O(NMAX),XCO2(NMAX),
     .            XCO(NMAX),XSUT(NMAX),N
      COMMON/TAUCOL/TAUIN(NMAX+1)
      COMMON /PLANK/C1,C2
      COMMON /RSPC/WVNB,DWVNB
      COMMON /LINDX/LICO,LICO2,LIH2O
      COMMON /INDEX/ICO,ICO2,IH2O
      COMMON /SPCD/DCO(14,48),DC(14,367),DH(14,367)
      COMMON /SPCK/KCO(14,48),KC(14,367),KH(14,367)
      COMMON /CFSUT/WV55
      COMMON /VALPI/PI
      DIMENSION XLUM(NMAX),XL(NWVMAX),XLS(NWVMAX),WV(NWVMAX)
C
C     READING THE COLUMN DATA FILE
C
      OPEN(UNIT=10,FILE='SNBINTENS')
      OPEN(UNIT=11,FILE='SNBTRANS')
      OPEN(UNIT=8,FILE='SNBINPUT')
      READ(8,*) N
      DO 1 I=1,N
      READ(8,*) XD(I),T(I),P(I),XH2O(I),XCO2(I),XCO(I),XSUT(I)
1     CONTINUE
      X=1.
      PI=4.0*DATAN(X)
C
C     READING THE MODEL PARAMETERS
C
      CALL PARAM
      MF=1
      NF=1
C
C     WAVE NUMBER LOOP
C
      IWVNB=0
      OPEN(UNIT=9,FILE='SNBWN')
1000  CONTINUE
      IWVNB=IWVNB+1
      READ(9,*) WVNB,DWVNB
      IF(WVNB.LT.0.) GOTO 1001
      WV55=WVNB*5.5
      WV(IWVNB)=WVNB
C
C     CALCULATION OF THE SPECTRAL INDEX FOR EACH SPECIES
C
      CALL FINDI
C
C     CALCULATION OF BLACKBODY INTENSITIES
C
      DO 3 I=1,N
      TT=T(I)
      XLUM(I)=XLU(TT)
3     CONTINUE
C
C     CALCULTATION OF TRANSMISSIVITIES
C
      CALL TRSMI
C
C     CALCULATION OF RADIATIVE INTENSITIES
C
      XL(IWVNB)=0.
      DO 4 I=1,N
      XL(IWVNB)=XL(IWVNB)+XLUM(I)*(TAUIN(I)-TAUIN(I+1))
4     CONTINUE
      XLS(IWVNB)=XL(IWVNB)/DWVNB
      WRITE(11,*) WVNB, TAUIN(N+1)
      WRITE(10,*) WVNB,XLS(IWVNB)
      GOTO 1000
1001  CONTINUE
      NWV=IWVNB-1
      XLT=0.
      DO 5 J=1,NWV
      XLT=XLT+XL(J)
5     CONTINUE
      WRITE(10,*) XLT
      REWIND(UNIT=9)
      CLOSE(UNIT=10)
      CLOSE(UNIT=11)
      CLOSE(UNIT=8)
      STOP
      END
C
C**************************************************************
C                                                             *
C     BLACKBODY INTENSITY (PLANCK FORMULA)                    *
C                                                             *
C**************************************************************
C
      DOUBLE PRECISION FUNCTION XLU(T)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /PLANK/C1,C2
      COMMON /VALPI/PI
      C2T=C2/T
      C2T=DEXP(C2T)-1.0
      XLU=C1/C2T
      RETURN
      END
C
C
C*****************************************************************
C                                                                *
C   THIS SUBROUTINE READS THE MODEL PARAMETERS                   *
C                                                                *
C*****************************************************************
C
      SUBROUTINE PARAM
      REAL KCO,KC,KH
      COMMON /SPCD/DCO(14,48),DC(14,367),DH(14,367)
      COMMON /SPCK/KCO(14,48),KC(14,367),KH(14,367)
      OPEN(UNIT=45,FILE='SNBCO')
      OPEN(UNIT=46,FILE='SNBCO2')
      OPEN(UNIT=47,FILE='SNBH2O')
C
C   READING THE PARAMETERS
C
      DO 1 I=1,48
1     READ(45,*) (KCO(J,I),J=1,12)
      DO 2 I=1,48
2     READ(45,*) (DCO(J,I),J=1,12)
      DO 3 I=1,96
3     READ(46,*) (KC(J,I),J=1,14)
      DO 4 I=1,96
4     READ(46,*) (DC(J,I),J=1,14)
      DO 5 I=1,367
5     READ(47,*) (KH(J,I),J=1,14)
      DO 6 I=1,367
6     READ(47,*) (DH(J,I),J=1,14)
      RETURN
      END
C
C
C***********************************************************************
C                                                                      *
C     THIS SUBROUTINE CALCULATES THE INTERPOLATION COEFFICIENTS        *
C                                                                      *
C***********************************************************************
C
      SUBROUTINE TMNO(TM,RT,IT)
      DOUBLE PRECISION TM,RT
      IF(TM.GT.300.0) THEN
         IF(TM.LT.2900.0) THEN
            RT=(TM-300.0)/200.0
            IT=INT(RT+1.0E-6)
            RT=RT-IT
            IT=IT+1
         ELSE
            RT=1.0
            IT=11
         ENDIF
      ELSE
         RT=0.0
         IT=1
      ENDIF
      RETURN
      END
C
C
C*******************************************************************
C                                                                  *
C     THIS SUBROUTINE SEARCHS THE PARAMETER INDEXES                *
C     CORRESPONDING THE WAVE NUMBER 'WVNB'                         *
C                                                                  *
C*******************************************************************
C
      SUBROUTINE FINDI
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      LOGICAL LICO,LICO2,LIH2O
      COMMON /LINDX/LICO,LICO2,LIH2O
      COMMON /INDEX/ICO,ICO2,IH2O
      COMMON /PLANK/C1,C2
      COMMON /RSPC/WVNB,DWVNB
C
C   CALCULATION OF PLANCK LAW PARAMETERS
C
      C1=(0.11909E-7)*WVNB**3*DWVNB
      C2=1.4388*WVNB
C
C   LOOKING FOR THE INDEXES
C
      IF(WVNB.LT.450.0.OR.WVNB.GT.1200) GO TO 10
      ICO2=INT((WVNB-450.)/25.)+1
      GO TO 14
10    IF(WVNB.LT.1950.0.OR.WVNB.GT.2450.) GO TO 11
      ICO2=INT((WVNB-1950.)/25.)+32
      GO TO 14
11    IF(WVNB.LT.3300.0.OR.WVNB.GT.3800.) GO TO 12
      ICO2=INT((WVNB-3300.)/25.)+53
      GO TO 14
12    IF(WVNB.LT.4700.0.OR.WVNB.GT.5250.) GO TO 13
      ICO2=INT((WVNB-4700.)/25.)+74
      GO TO 14
13    ICO2=-9
14    CONTINUE
      IF(WVNB.GE.150.AND.WVNB.LE.9300.) THEN
        IH2O=INT((WVNB-150.)/25.)+1
      ELSE
        IH2O=-9
      ENDIF
      IF(WVNB.LT.1750.0.OR.WVNB.GT.2325.) GO TO 19
      ICO=INT((WVNB-1750.)/25.)+1
      GO TO 21
19    IF(WVNB.LT.3775.0.OR.WVNB.GT.4350.) GO TO 20
      ICO=INT((WVNB-3775.)/25.)+25
      GO TO 21
20    ICO=-9
21    CONTINUE
      LICO=ICO.GT.0
      LICO2=ICO2.GT.0
      LIH2O=IH2O.GT.0
      RETURN
      END
C
C
C***********************************************************
C                                                          *
C     CALCULATION OF THE TRANSMISSIVITY OF A INHOMOGENEOUS *
C     COLUMN WITH CURTIS-GODSON APPROXIMATION              *
C     TRANSMISSIVITIES ARE CALCULATED BETWEEN THE FIRST    *
C     POINT AND A CURRENT POINT OF THE COLUMN              *
C                                                          *
C***********************************************************
C
C
      SUBROUTINE TRSMI
      IMPLICIT DOUBLE PRECISION (A-C,E-H,O-Z)
      REAL KCO,KC,KH
      LOGICAL LICO,LICO2,LIH2O
      PARAMETER (NMAX=100)
      COMMON/PHYS/XD(NMAX),T(NMAX),P(NMAX),XH2O(NMAX),XCO2(NMAX),
     .            XCO(NMAX),XSUT(NMAX),N
      COMMON /LINDX/LICO,LICO2,LIH2O
      COMMON /SPCD/DCO(14,48),DC(14,367),DH(14,367)
      COMMON /SPCK/KCO(14,48),KC(14,367),KH(14,367)
      COMMON /INDEX/ICO,ICO2,IH2O
      COMMON /CFSUT/WV55
      COMMON/TAUCOL/TAUIN(NMAX+1)
      DIMENSION XKCO(NMAX),XDCO(NMAX),XBCO(NMAX)
      DIMENSION XKCO2(NMAX),XDCO2(NMAX),XBCO2(NMAX)
      DIMENSION XKH2O(NMAX),XDH2O(NMAX),XBH2O(NMAX)
      DIMENSION RRT(NMAX),IIT(NMAX),XH(NMAX)
                                            
      NCOL=N+1
      DO 1 J=1,NCOL-1
      XH(J)=XD(J)
      CALL TMNO(T(J),RT,IT)
      RRT(J)=RT
      IIT(J)=IT
      T296=296./T(J)
      T273=273./T(J)
      T900=900./T(J)
      XN2=1.-XCO(J)-XCO2(J)-XH2O(J)
      IF(LICO) THEN
        GAM=0.07*XCO2(J)+0.06*(XCO(J)+XN2+XH2O(J))
        GAM=P(J)*GAM*SQRT(T273)
        XKCO(J)=KCO(IT,ICO)+RT*(KCO(IT+1,ICO)-KCO(IT,ICO))
        XDCO(J)=DCO(IT,ICO)+RT*(DCO(IT+1,ICO)-DCO(IT,ICO))
        XBCO(J)=2.*GAM*XDCO(J)
      ENDIF
      IF(LICO2) THEN
        GAM=0.07*XCO2(J)+0.058*XN2+0.15*XH2O(J)
        IF(T(J).LE.900.) THEN
        GAM=P(J)*GAM*(T296)**0.7
        ELSE
        GAM=P(J)*GAM*0.45913*DSQRT(T900)
        ENDIF
        XKCO2(J)=KC(IT,ICO2)+RT*(KC(IT+1,ICO2)-KC(IT,ICO2))
        XDCO2(J)=DC(IT,ICO2)+RT*(DC(IT+1,ICO2)-DC(IT,ICO2))
        XBCO2(J)=2.*GAM*XDCO2(J)
      ENDIF
      IF(LIH2O) THEN
        RATT=DSQRT(T296)
        GAM=0.066*(7.0*RATT*XH2O(J)+1.2*(XH2O(J)+XN2)+1.5*XCO2(J))*RATT
        GAM=P(J)*GAM
        XKH2O(J)=KH(IT,IH2O)+RT*(KH(IT+1,IH2O)-KH(IT,IH2O))
        XDH2O(J)=DH(IT,IH2O)+RT*(DH(IT+1,IH2O)-DH(IT,IH2O))
        XBH2O(J)=2.*GAM*XDH2O(J)
      ENDIF
1     CONTINUE
C
C     THE TRANSMISSIVITIES ARE INITIALIZED WITH THE TRANSMISSIVITY
C     OF THE PARTICLES.
C
      TAUIN(1)=1.
      DO 12 I=1,NCOL-1
      HSUIE=0.
      DO 12 J=2,NCOL
      HSUIE=HSUIE-XH(J-1)*WV55*XSUT(J-1)
      TAUIN(J)=DEXP(HSUIE)
12    CONTINUE             
      XMCO=0.
      XMCO2=0.
      XMH2O=0.
      DO 4 I=1,NCOL-1
      HT=HT+XH(I)
      XMCO=XMCO+XCO(I)
      XMCO2=XMCO2+XCO2(I)
      XMH2O=XMH2O+XH2O(I)
4     CONTINUE
C
C     CALCULATION OF CO TRANSMISSIVITY
C
      IF(LICO.AND.XMCO.GT.1.0E-6) THEN
      SC=0.
      SK=0.
      SB=0.
      DO 3 J=2,NCOL
      YC=XCO(J-1)*P(J-1)*XH(J-1)
      YK=YC*XKCO(J-1)
      YB=YK*XBCO(J-1)
      SC=SC+YC
      IF(SC.GT.1.D-12) THEN
      SK=SK+YK
      SB=SB+YB
      ZC=SC
      ZK=SK/ZC
      ZB=SB/SK
      XK=1.0+2.0*ZK*ZC/ZB
      TAUIN(J)=TAUIN(J)*DEXP(-ZB*(DSQRT(XK)-1.0))
      ENDIF
3     CONTINUE
      ENDIF
C
C     CALCULATION OF CO2 TRANSMISSIVITY
C
      IF(LICO2.AND.XMCO2.GT.1.0E-6) THEN
      SC=0.
      SK=0.
      SB=0.
      DO 5 J=2,NCOL
      YC=XCO2(J-1)*P(J-1)*XH(J-1)
      YK=YC*XKCO2(J-1)
      YB=YK*XBCO2(J-1)
      SC=SC+YC
      IF(SC.GT.1.D-12.AND.YK.GT.1.D-15) THEN
      SK=SK+YK
      SB=SB+YB
      ZC=SC
      ZK=SK/ZC
      ZB=SB/SK
      XK=1.0+2.0*ZK*ZC/ZB
      TAUIN(J)=TAUIN(J)*DEXP(-ZB*(DSQRT(XK)-1.0))
      ENDIF
5     CONTINUE
      ENDIF
C
C     CALCULATION OF H2O TRANSMISSIVITY
C
      IF(LIH2O.AND.XMH2O.GT.1.0E-6) THEN
      SC=0.
      SK=0.
      SB=0.
      DO 6 J=2,NCOL
      YC=XH2O(J-1)*P(J-1)*XH(J-1)
      YK=YC*XKH2O(J-1)
      YB=YK*XBH2O(J-1)
      SC=SC+YC
      IF(SC.GT.1.D-12) THEN
      SK=SK+YK
      SB=SB+YB
      ZC=SC
      ZK=SK/ZC
      ZB=SB/SK
      XK=1.0+2.0*ZK*ZC/ZB
      TAUIN(J)=TAUIN(J)*DEXP(-ZB*(DSQRT(XK)-1.0))
      ENDIF
6     CONTINUE
      ENDIF
      RETURN
      END
